import json
import mock

import pytest


def test_pytest_messenger_slack_failed(testdir):
    """Make sure that our pytest-messenger works."""

    testdir.makepyfile(
        """
        import pytest
        def test_pass():
            assert 1 == 1


        def test_fail():
            assert 1 == 2


        @pytest.mark.skip()
        def test_skip():
            assert 1 == 1


        def test_error(test):
            assert 1 == ""


        @pytest.mark.xfail()
        def test_xfail():
            assert 1 == 2

        @pytest.mark.xfail()
        def test_xpass():
            assert 1 == 1
        """
    )

    slack_hook_host = 'http://test.com/any_hash'
    slack_hook_username = 'regression Testing'
    slack_hook_report_host = 'http://report_link.com'
    slack_hook_channel = 'test'
    slack_hook_icon_emoji = ':thumbsdown:'
    expected_text = '<http://report_link.com|Passed=1 Failed=1 Skipped=1 Error=1 XFailed=1 XPassed=1>'
    with mock.patch('requests.post') as mock_post:
        testdir.runpytest('--slack_channel', slack_hook_channel,
                          '--slack_hook', slack_hook_host,
                          '--slack_report_link', slack_hook_report_host,
                          '--slack_username', slack_hook_username,
                          '--slack_failed_emoji', slack_hook_icon_emoji)

        called_data = json.loads(mock_post.call_args[1]['data'])
        called_host = mock_post.call_args[0][0]
        called_channel = called_data['channel']
        called_username = called_data['username']
        text = called_data['attachments'][0]['text']
        color = called_data['attachments'][0]['color']
        emoji = called_data['icon_emoji']

        assert called_host == slack_hook_host
        assert text == expected_text
        assert called_channel == slack_hook_channel
        assert called_username == slack_hook_username
        assert color == '#ff0000'
        assert emoji == slack_hook_icon_emoji


def test_pytest_messenger_slack_passed(testdir):
    """Make sure that our pytest-messenger works."""

    testdir.makepyfile(
        """
        import pytest
        def test_pass():
            assert 1 == 1

        """
    )

    slack_hook_host = 'http://test.com/any_hash'
    slack_hook_username = 'regression Testing'
    slack_hook_report_host = 'http://report_link.com'
    slack_hook_channel = 'test'
    slack_hook_icon_emoji = ':thumbsup:'
    expected_text = '<http://report_link.com|Passed=1 Failed=0 Skipped=0 Error=0 XFailed=0 XPassed=0>'
    with mock.patch('requests.post') as mock_post:
        testdir.runpytest('--slack_channel', slack_hook_channel,
                          '--slack_hook', slack_hook_host,
                          '--slack_report_link', slack_hook_report_host,
                          '--slack_username', slack_hook_username,
                          '--slack_success_emoji', slack_hook_icon_emoji)

        called_data = json.loads(mock_post.call_args[1]['data'])
        called_host = mock_post.call_args[0][0]
        called_channel = called_data['channel']
        called_username = called_data['username']
        text = called_data['attachments'][0]['text']
        color = called_data['attachments'][0]['color']
        emoji = called_data['icon_emoji']

        assert called_host == slack_hook_host
        assert text == expected_text
        assert called_channel == slack_hook_channel
        assert called_username == slack_hook_username
        assert color == '#56a64f'
        assert emoji == slack_hook_icon_emoji


@pytest.mark.parametrize('expected_prefix,report_link', [
    ("Test Prefix", None),
    ("Test Prefix", "http://report_link.com")
])
def test_pytest_messenger_slack_message_prefix(testdir, expected_prefix, report_link):
    """Make sure that message prefix works."""

    testdir.makepyfile(
        """
        import pytest
        def test_pass():
            assert 1 == 1
        """
    )

    slack_hook_host = 'http://test.com/any_hash'
    slack_hook_channel = 'test'
    run_args = [
        '--slack_channel', slack_hook_channel,
        '--slack_hook', slack_hook_host,
        '--slack_message_prefix', expected_prefix
    ]
    if report_link:
        run_args.extend(['--slack_report_link', report_link])

    expected_text = 'Passed=1 Failed=0 Skipped=0 Error=0 XFailed=0 XPassed=0'
    if report_link:
        expected_text = '<%s|%s>' % (report_link, expected_text)
    expected_text = '%s: %s' % (expected_prefix, expected_text)

    with mock.patch('requests.post') as mock_post:
        testdir.runpytest(*run_args)

        called_data = json.loads(mock_post.call_args[1]['data'])
        text = called_data['attachments'][0]['text']

        assert text == expected_text


@pytest.mark.parametrize('test_input,expected_emoji', [
    ('1 == 1', ':sunny:'),
    ('2 == 1', ':rain_cloud:'),
])
def test_pytest_messenger_slack_custom_emojis(testdir, test_input, expected_emoji):
    testdir.makepyfile(
        """
        def test_icon_emoji():
            assert %s
        """ % test_input
    )

    slack_hook_host = 'http://test.com/any_hash'
    slack_hook_channel = 'test'
    slack_hook_icon_emoji_success = ':sunny:'
    slack_hook_icon_emoji_failed = ':rain_cloud:'
    with mock.patch('requests.post') as mock_post:
        testdir.runpytest('--slack_channel', slack_hook_channel,
                          '--slack_hook', slack_hook_host,
                          '--slack_success_emoji', slack_hook_icon_emoji_success,
                          '--slack_failed_emoji', slack_hook_icon_emoji_failed)

        called_data = json.loads(mock_post.call_args[1]['data'])
        emoji = called_data['icon_emoji']

        assert emoji == expected_emoji


@pytest.mark.parametrize('test_input,expected_url', [
    ('1 == 1', 'http://localhost/success.png'),
    ('2 == 1', 'http://localhost/failed.png'),
])
def test_pytest_messenger_slack_custom_icons(testdir, test_input, expected_url):
    testdir.makepyfile(
        """
        def test_icon_url():
            assert %s
        """ % test_input
    )

    slack_hook_host = 'http://test.com/any_hash'
    slack_hook_channel = 'test'
    slack_hook_icon_url_success = 'http://localhost/success.png'
    slack_hook_icon_url_failed = 'http://localhost/failed.png'
    with mock.patch('requests.post') as mock_post:
        testdir.runpytest('--slack_channel', slack_hook_channel,
                          '--slack_hook', slack_hook_host,
                          '--slack_success_icon', slack_hook_icon_url_success,
                          '--slack_failed_icon', slack_hook_icon_url_failed)

        called_data = json.loads(mock_post.call_args[1]['data'])
        emoji = called_data['icon_url']

        assert emoji == expected_url


@pytest.mark.parametrize('test_input,expected_url', [
    ('1 == 1', 'http://localhost/success.png'),
    ('2 == 1', 'http://localhost/failed.png'),
])
def test_pytest_messenger_slack_icon_overrides_emoji(testdir, test_input, expected_url):
    testdir.makepyfile(
        """
        def test_icon_url():
            assert %s
        """ % test_input
    )

    slack_hook_host = 'http://test.com/any_hash'
    slack_hook_channel = 'test'
    slack_hook_icon_url_success = 'http://localhost/success.png'
    slack_hook_icon_url_failed = 'http://localhost/failed.png'
    slack_hook_icon_emoji_success = ':sunny:'
    slack_hook_icon_emoji_failed = ':rain_cloud:'
    with mock.patch('requests.post') as mock_post:
        testdir.runpytest('--slack_channel', slack_hook_channel,
                          '--slack_hook', slack_hook_host,
                          '--slack_success_icon', slack_hook_icon_url_success,
                          '--slack_failed_icon', slack_hook_icon_url_failed,
                          '--slack_success_emoji', slack_hook_icon_emoji_success,
                          '--slack_failed_emoji', slack_hook_icon_emoji_failed)

        called_data = json.loads(mock_post.call_args[1]['data'])
        url = called_data['icon_url']
        emoji = called_data['icon_emoji']

        assert url == expected_url
        assert emoji is None



import base64
import hashlib
import hmac
import time
import urllib.parse
import json
import requests


def add_ding_options(parser):
    group = parser.getgroup('pytest-messenger[ding]')
    group.addoption(
        '--ding_access_token',
        action='store',
        dest='ding_access_token',
        default=None
    )
    group.addoption(
        '--ding_secret',
        action='store',
        dest='ding_secret',
        default=None,
    )

    group.addoption(
        '--ding_report_link',
        action='store',
        dest='ding_report_link',
        default=None,
        help='Set the report link'
    )

    group.addoption(
        '--ding_timeout',
        action='store',
        dest='ding_timeout',
        default=10,
        help='Set the report send timeout'
    )


def ding_send_message(test_result, config, exitstatus):
    report_link = config.option.ding_report_link
    secret = config.option.ding_secret
    access_token = config.option.ding_access_token

    secret_enc = secret.encode('utf-8')
    timestamp = str(round(time.time() * 1000))
    string_to_sign = '{}\n{}'.format(timestamp, secret)
    string_to_sign_enc = string_to_sign.encode('utf-8')
    hmac_code = hmac.new(secret_enc, string_to_sign_enc, digestmod=hashlib.sha256).digest()
    sign = urllib.parse.quote_plus(base64.b64encode(hmac_code))
    timeout = config.option.ding_timeout

    final_results = 'Passed=%s Failed=%s Skipped=%s Error=%s XFailed=%s XPassed=%s' % (
        test_result.passed,
        test_result.failed,
        test_result.skipped,
        test_result.error,
        test_result.xfailed,
        test_result.xpassed)
    if report_link:
        final_results = '<%s | %s>' % (report_link, final_results)
    payload = {
        "msgtype": "text",
        "text": {
            "content": final_results
        },

    }

    requests.post(f"https://oapi.dingtalk.com/robot/send?access_token={access_token}&timestamp={timestamp}&sign={sign}",
                  json=payload, timeout=timeout)



import requests
import json


def add_slack_options(parser):
    group = parser.getgroup('pytest-messenger[slack]')
    group.addoption(
        '--ssl_verify',
        action='store',
        dest='ssl_verify',
        default=True,
        help='Set the TLS certificate verification'
    )
    group.addoption(
        '--slack_channel',
        action='store',
        dest='slack_channel',
        default=None,
        help='Set the channel name to report'
    )
    group.addoption(
        '--slack_hook',
        action='store',
        dest='slack_hook',
        default=None,
        help='Used for reporting to slack'
    )

    group.addoption(
        '--slack_report_link',
        action='store',
        dest='slack_report_link',
        default=None,
        help='Set the report link'
    )

    group.addoption(
        '--slack_username',
        action='store',
        dest='slack_username',
        default=None,
        help='Set the reporter name'
    )

    group.addoption(
        '--slack_message_prefix',
        action='store',
        dest='slack_message_prefix',
        default=None,
        help='Set a prefix to come before the test result counts.'
    )

    group.addoption(
        '--slack_timeout',
        action='store',
        dest='slack_timeout',
        default=10,
        help='Set the report send timeout'
    )

    group.addoption(
        '--slack_success_emoji',
        action='store',
        dest='slack_success_emoji',
        default=':thumbsup:',
        help='Set emoji for a successful run'
    )

    group.addoption(
        '--slack_failed_emoji',
        action='store',
        dest='slack_failed_emoji',
        default=':thumbsdown:',
        help='Set emoji for a failed run'
    )

    group.addoption(
        '--slack_success_icon',
        action='store',
        dest='slack_success_icon',
        default=None,
        help='Set icon (a url) for a successful run. Overrides slack_success_emoji'
    )

    group.addoption(
        '--slack_failed_icon',
        action='store',
        dest='slack_failed_icon',
        default=None,
        help='Set icon (a url) for a failed run. Overrides slack_failed_icon'
    )


def slack_send_message(test_result, config, exitstatus):
    timeout = config.option.slack_timeout
    report_link = config.option.slack_report_link
    slack_hook = config.option.slack_hook
    channel = config.option.slack_channel
    ssl_verify = config.option.ssl_verify
    message_prefix = config.option.slack_message_prefix
    slack_username = config.option.slack_username if config.option.slack_username else 'Regression testing results'
    if int(exitstatus) == 0:
        color = "#56a64f"
        emoji = config.option.slack_success_emoji
        icon = config.option.slack_success_icon
    else:
        color = '#ff0000'
        emoji = config.option.slack_failed_emoji
        icon = config.option.slack_failed_icon
    final_results = 'Passed=%s Failed=%s Skipped=%s Error=%s XFailed=%s XPassed=%s' % (
        test_result.passed,
        test_result.failed,
        test_result.skipped,
        test_result.error,
        test_result.xfailed,
        test_result.xpassed)
    if report_link:
        final_results = '<%s|%s>' % (report_link, final_results)
    if message_prefix:
        final_results = '%s: %s' % (message_prefix, final_results)
    results_pattern = {
        "color": color,
        "text": final_results,
        "mrkdwn_in": [
            "text",
            "pretext"
        ]
    }
    payload = {"channel": channel,
               "username": slack_username,
               "attachments": [results_pattern],
               "icon_emoji": emoji if icon is None else None,
               "icon_url": icon}
    requests.post(slack_hook, data=json.dumps(payload), timeout=timeout, verify=ssl_verify)



import json
import mock

import pytest


def test_pytest_messenger_ding_failed(testdir):
    """Make sure that our pytest-messenger works."""

    testdir.makepyfile(
        """
        import pytest
        def test_pass():
            assert 1 == 1


        def test_fail():
            assert 1 == 2


        @pytest.mark.skip()
        def test_skip():
            assert 1 == 1


        def test_error(test):
            assert 1 == ""


        @pytest.mark.xfail()
        def test_xfail():
            assert 1 == 2

        @pytest.mark.xfail()
        def test_xpass():
            assert 1 == 1
        """
    )

    ding_secret = 'SuperSecret'
    ding_access = 'UnlimitedAccess'
    report_link = 'http://report_link.com'
    expected_text = '<http://report_link.com | Passed=1 Failed=1 Skipped=1 Error=1 XFailed=1 XPassed=1>'
    with mock.patch('requests.post') as mock_post:
        testdir.runpytest('--ding_secret', ding_secret,
                          '--ding_access_token', ding_access,
                          '--ding_report_link', report_link)

        called_data = mock_post.call_args[1]['json']
        text = called_data['text']['content']

        assert text == expected_text


def test_pytest_messenger_ding_passed(testdir):
    """Make sure that our pytest-messenger works."""

    testdir.makepyfile(
        """
        import pytest
        def test_pass():
            assert 1 == 1

        """
    )

    ding_secret = 'SuperSecret'
    ding_access = 'UnlimitedAccess'
    report_link = 'http://report_link.com'
    expected_text = '<http://report_link.com | Passed=1 Failed=0 Skipped=0 Error=0 XFailed=0 XPassed=0>'
    with mock.patch('requests.post') as mock_post:
        testdir.runpytest('--ding_secret', ding_secret,
                          '--ding_access_token', ding_access,
                          '--ding_report_link', report_link)

        called_data = mock_post.call_args[1]['json']
        text = called_data['text']['content']

        assert text == expected_text



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open('AUTHORS.rst') as authors_file:
    authors = authors_file.read()

requirements = ['requests']

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest', ]

setup(
    author="LaserPhaser",
    author_email='arseny.antonov@gmail.com',
    classifiers=[
        'Framework :: Pytest',
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Pytest to Slack reporting plugin",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history + '\n\n' + authors,
    include_package_data=True,
    keywords=[
        'pytest', 'py.test', 'slack',
    ],
    name='pytest-messenger',
    packages=find_packages(include=['pytest_messenger']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/pytest-dev/pytest-messenger',
    version='3.1.0',
    zip_safe=False,
    entry_points={
        'pytest11': [
            'pytest-messenger = pytest_messenger.plugin',
        ]
    }
)
